import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import bcrypt from 'bcrypt';
import { Repository } from 'typeorm';
import { AccountsEntity } from '../../database/entities';

@Injectable()
export class UsersAuthService {
  constructor(
    @InjectRepository(AccountsEntity)
    private readonly accountsRepository: Repository<AccountsEntity>
  ) {}

  async getUserById(accountId: number) {
    return await this.accountsRepository.findOne({
      relations: { profili: { ruolo: true } },
      where: { id: accountId },
    });
  }

  async getUserByUsername(username: string) {
    return await this.accountsRepository.findOne({
      relations: { profili: { ruolo: true } },
      where: { username: username },
    });
  }

  async getUserByIdAndRefreshTokenPair(accountId: number, refreshToken: string) {
    const accountById = await this.getUserById(accountId);
    if (!accountById?.ultimoHashRefreshToken) return null;

    const isRefreshTokenMatching = await bcrypt.compare(refreshToken, accountById.ultimoHashRefreshToken);
    return isRefreshTokenMatching ? accountById : null;
  }

  async setCurrentRefreshTokenHash(accountId: number, refreshToken: string | null) {
    const hash = refreshToken ? await bcrypt.hash(refreshToken, 10) : null;
    await this.accountsRepository.update(accountId, {
      ultimoHashRefreshToken: hash,
    });
  }
}
